searchData={"items":[{"type":"module","doc":"ExFdbmonitor is an Elixir application that manages the starting and stopping of\n`fdbmonitor`, which is the management process for FoundationDB.\n\nThe goal of ExFdbmonitor is to allow a FoundationDB cluster to bootstrap itself\nusing the distributed capabilities of the Erlang VM.\n\nWith a correctly crafted set of application environment variables, a cluster\ncan be brought up from zero as long as each node is started individually.\n\nOnce the cluster is established, node restarts are equivalent to restarts of\n`fdbmonitor` itself.","title":"ExFdbmonitor","ref":"ExFdbmonitor.html"},{"type":"module","doc":"","title":"Configuration of `:ex_fdbmonitor` - ExFdbmonitor","ref":"ExFdbmonitor.html#module-configuration-of-ex_fdbmonitor"},{"type":"module","doc":"If your FoundationDB installation is not in the default location, then you must set\nthe following environment variables. The paths shown here are the defaults.\n\n```\nconfig :ex_fdbmonitor,\n       fdbcli: \"/usr/local/bin/fdbcli\",\n       fdbserver: \"/usr/local/libexec/fdbserver\",\n       fdbdr: \"/usr/local/bin/fdbdr\",\n       backup_agent: \"/usr/local/foundationdb/backup_agent/backup_agent\",\n       dr_agent: \"/usr/local/bin/dr_agent\"\n```","title":"FDB executable paths - ExFdbmonitor","ref":"ExFdbmonitor.html#module-fdb-executable-paths"},{"type":"module","doc":"The env vars `:etc_dir` and `:run_dir` are used on every boot.\n\n```elixir\ndatabase_path = \"/var/lib/myapp/data/fdb\"\n\nconfig :ex_fdbmonitor,\n  etc_dir: Path.join(database_path, \"etc\"),\n  run_dir: Path.join(database_path, \"run\")\n```\n\nThe `:bootstrap` env var is used only on first boot of each node in the cluster.\nOnce a cluster is established, it is ignored on all subsequent boots. A simple\nexample is shown here.\n\n```elixir\nconfig :ex_fdbmonitor,\n  bootstrap: [\n    cluster: [\n      coordinator_addr: \"127.0.0.1\"\n    ],\n    conf: [\n      data_dir: Path.join(database_path, \"data\"),\n      log_dir: Path.join(database_path, \"log\"),\n      fdbservers: [\n        [port: 5000]\n      ]\n    ],\n    fdbcli: ~w[configure new single ssd-redwood-1]\n  ]\n```","title":"FDB cluster configuration - ExFdbmonitor","ref":"ExFdbmonitor.html#module-fdb-cluster-configuration"},{"type":"module","doc":"See [examples/example_app/README.md](example-app.html) for a tutorial on\nusing ExFdbmonitor in your application.","title":"Usage - ExFdbmonitor","ref":"ExFdbmonitor.html#module-usage"},{"type":"function","doc":"Opens a database connection.","title":"ExFdbmonitor.open_db/1","ref":"ExFdbmonitor.html#open_db/1"},{"type":"function","doc":"- `:input`: Ignored. It's here for compatibility with other libraries.","title":"Arguments - ExFdbmonitor.open_db/1","ref":"ExFdbmonitor.html#open_db/1-arguments"},{"type":"function","doc":"Use whenever you need to open the `t:erlfdb.database/0`:\n\n```elixir\ndb = ExFdbmonitor.open_db()\n\"world\" = :erlfdb.get(db, \"hello\")\n```\n\nUse in conjuncation with `Ecto.Adapters.FoundationDB`:\n\n```elixir\nconfig :my_app, MyApp.Repo,\n  open_db: &ExFdbmonitor.open_db/1\n```","title":"Examples - ExFdbmonitor.open_db/1","ref":"ExFdbmonitor.html#open_db/1-examples"},{"type":"module","doc":"This module provides functions for executing fdbcli commands.","title":"ExFdbmonitor.Fdbcli","ref":"ExFdbmonitor.Fdbcli.html"},{"type":"function","doc":"","title":"ExFdbmonitor.Fdbcli.exec/1","ref":"ExFdbmonitor.Fdbcli.html#exec/1"},{"type":"function","doc":"","title":"ExFdbmonitor.Fdbcli.exec/2","ref":"ExFdbmonitor.Fdbcli.html#exec/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Fdbcli.exec/3","ref":"ExFdbmonitor.Fdbcli.html#exec/3"},{"type":"module","doc":"This module provides functions for executing fdbdr commands.","title":"ExFdbmonitor.Fdbdr","ref":"ExFdbmonitor.Fdbdr.html"},{"type":"function","doc":"","title":"ExFdbmonitor.Fdbdr.exec/3","ref":"ExFdbmonitor.Fdbdr.html#exec/3"},{"type":"module","doc":"Build a sandbox FoundationDB cluster.","title":"ExFdbmonitor.Sandbox","ref":"ExFdbmonitor.Sandbox.html"},{"type":"module","doc":"```elixir\nalias ExFdbmonitor.Sandbox\n\nSandbox.start()\ncluster_file = Sandbox.cluster_file(\"dev\", 0)\nsandbox = Sandbox.Single.checkout(\"dev\", starting_port: 5050)\nIO.puts(\"Cluster file: #{cluster_file}\")\n_anything = IO.gets(\"Input anything to stop FDB: \")\nSandbox.Single.checkin(sandbox)\n```","title":"Examples - ExFdbmonitor.Sandbox","ref":"ExFdbmonitor.Sandbox.html#module-examples"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.build_context/3","ref":"ExFdbmonitor.Sandbox.html#build_context/3"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.checkin/2","ref":"ExFdbmonitor.Sandbox.html#checkin/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.checkout/3","ref":"ExFdbmonitor.Sandbox.html#checkout/3"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.cluster_file/1","ref":"ExFdbmonitor.Sandbox.html#cluster_file/1"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.cluster_file/2","ref":"ExFdbmonitor.Sandbox.html#cluster_file/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.data_dir/2","ref":"ExFdbmonitor.Sandbox.html#data_dir/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.etc_dir/2","ref":"ExFdbmonitor.Sandbox.html#etc_dir/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.log_dir/2","ref":"ExFdbmonitor.Sandbox.html#log_dir/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.nodes/1","ref":"ExFdbmonitor.Sandbox.html#nodes/1"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.run_dir/2","ref":"ExFdbmonitor.Sandbox.html#run_dir/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.start/0","ref":"ExFdbmonitor.Sandbox.html#start/0"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.stop/0","ref":"ExFdbmonitor.Sandbox.html#stop/0"},{"type":"module","doc":"This module provides functions for managing multi-node FoundationDB sandboxes in double redundancy mode.","title":"ExFdbmonitor.Sandbox.Double","ref":"ExFdbmonitor.Sandbox.Double.html"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.Double.checkin/2","ref":"ExFdbmonitor.Sandbox.Double.html#checkin/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.Double.checkout/2","ref":"ExFdbmonitor.Sandbox.Double.html#checkout/2"},{"type":"module","doc":"This module provides functions for managing single-node FoundationDB sandboxes.","title":"ExFdbmonitor.Sandbox.Single","ref":"ExFdbmonitor.Sandbox.Single.html"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.Single.checkin/2","ref":"ExFdbmonitor.Sandbox.Single.html#checkin/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.Single.checkout/2","ref":"ExFdbmonitor.Sandbox.Single.html#checkout/2"},{"type":"module","doc":"This module provides functions for managing multi-node FoundationDB sandboxes in triple redundancy mode.","title":"ExFdbmonitor.Sandbox.Triple","ref":"ExFdbmonitor.Sandbox.Triple.html"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.Triple.checkin/2","ref":"ExFdbmonitor.Sandbox.Triple.html#checkin/2"},{"type":"function","doc":"","title":"ExFdbmonitor.Sandbox.Triple.checkout/2","ref":"ExFdbmonitor.Sandbox.Triple.html#checkout/2"},{"type":"extras","doc":"# ExampleApp\n\nThis is an example application that demonstrates how to use `ex_fdbmonitor` in both\ndevelopment and production along with `ecto_foundationdb`. Together, these two\ndependencies can be a basis for how your application state is managed across different\nenvironments.\n\nThis README contains the comprehensive list of steps taken to construct this example, so\nthat the reader may follow along with their own application.","title":"ExampleApp","ref":"example-app.html"},{"type":"extras","doc":"","title":"Guide - ExampleApp","ref":"example-app.html#guide"},{"type":"extras","doc":"From [FoundationDB Releases](https://github.com/apple/foundationdb/releases), choose the latest\nstable release, and install the `foundationdb-clients` and `foundationdb-server` packages for your\nsystem.\n\nAt the time of writing, a MacOS system should use [FoundationDB-7.3.42_arm64.pkg](https://github.com/apple/foundationdb/releases/download/7.3.42/FoundationDB-7.3.42_arm64.pkg)\nto install both `foundationdb-clients` and `foundationdb-server`.","title":"Install system dependencies - ExampleApp","ref":"example-app.html#install-system-dependencies"},{"type":"extras","doc":"```bash\nmix new example_app --sup\ncd example_app\n```\n\n```bash\n# .gitignore\n/.example_app/\n/.erlfdb_sandbox/\n```","title":"Create the application with mix - ExampleApp","ref":"example-app.html#create-the-application-with-mix"},{"type":"extras","doc":"This section describes a conventional way to manage application config in differing\nmix environments. For now, we'll leave the config files empty, and we'll fill them in\nlater.\n\n```bash\nmkdir config\ntouch config/dev.exs config/test.exs config/prod.exs\n```\n\n```elixir\n# config/config.exs\nimport Config\n\nimport_config \"#{config_env()}.exs\"\n```","title":"Set up application config - ExampleApp","ref":"example-app.html#set-up-application-config"},{"type":"extras","doc":"```elixir\n  # mix.exs\n  defp deps do\n  [\n    {:ecto_foundationdb, \"~> 0.4\"},\n    {:ex_fdbmonitor, \"~> 0.1\", only: :dev}\n  ]\n  end\n```\n\n```bash\nmix deps.get\nmix\n```\n\nYou'll see a warning, which we'll address in the next step.\n\n```\n10:14:17.299 [warning] ExFdbmonitor starting without running fdbmonitor. At minimum, you should define `:etc_dir` and `:run_dir`, but you should also consider adding a `:bootstrap` config.\n```","title":"Add `:ecto_foundationdb` and `:ex_fdbmonitor` deps - ExampleApp","ref":"example-app.html#add-ecto_foundationdb-and-ex_fdbmonitor-deps"},{"type":"extras","doc":"The dev.exs configuration shown here creates a working directory for FoundationDB\nthat is expected to be semi-permanent. That is, this FoundationDB cluster (of 1 node)\ncan be kept long term during development of your application. It will be in active\nuse when you run `iex -S mix`.\n\nYou can modify the directories and port(s) according to your needs.\n\nNote: This is not a \"Sandbox\". Sandboxes should be reserved for\ndatabases that are ephemeral and safe to delete at any time.\n\n#### Configure ex_fdbmonitor\n\nWhen your app is running with this config, it will start a single fdbserver process\nand it will listen on port 5000. It uses the ssd-redwood-1 storage engine.\n\n```elixir\n# config/dev.exs\nimport Config\n\nconfig :ex_fdbmonitor,\n  etc_dir: \".example_app/dev/fdb/etc\",\n  run_dir: \".example_app/dev/fdb/run\"\n\nconfig :ex_fdbmonitor,\n  bootstrap: [\n    cluster: [\n      coordinator_addr: \"127.0.0.1\"\n    ],\n    conf: [\n      data_dir: \".example_app/dev/fdb/data\",\n      log_dir: \".example_app/dev/fdb/log\",\n      fdbservers: [\n        [port: 5000]\n      ]\n    ],\n    fdbcli: ~w[configure new single ssd-redwood-1]\n  ]\n```\n\n#### Verify dev FDB\n\nTo verify this config, you can run\n\n```bash\niex -S mix\n```\n\nand then\n\n```elixir\niex(1)> ExFdbmonitor.Fdbcli.exec(\"status minimal\")\n{:ok, [stdout: [\"The database is available.\\n\"]]}\n```\n\nwhich is equivalent to\n\n```bash\nfdbcli -C .example_app/dev/fdb/etc/fdb.cluster --exec \"status minimal\"\n```","title":"For `MIX_ENV=dev` - ExampleApp","ref":"example-app.html#for-mix_env-dev"},{"type":"extras","doc":"Before setting up `mix test`, let's make sure our app can interact with a database\nvia `Ecto`.\n\nAdd a repo.ex file:\n\n```elixir\n# lib/example_app/repo.ex\ndefmodule ExampleApp.Repo do\n  use Ecto.Repo, otp_app: :example_app, adapter: Ecto.Adapters.FoundationDB\n\n  use EctoFoundationDB.Migrator\n\n  def migrations(), do: []\nend\n```\n\nAdd this section to dev.exs:\n\n```elixir\n# config/dev.exs\nconfig :example_app, ExampleApp.Repo,\n  open_db: &ExFdbmonitor.open_db/1\n# ...\n```\n\nStart the Repo in your application:\n\n```elixir\n# lib/example_app/application.ex\ndefmodule ExampleApp.Application do\n  # ...\n  def start(_type, _args) do\n    children = [\n      ExampleApp.Repo\n    ]\n\n    # ...\n  end\nend\n```\n\nYour `dev` environment is now ready to use.","title":"Creating `ExampleApp.Repo` - ExampleApp","ref":"example-app.html#creating-exampleapp-repo"},{"type":"extras","doc":"In the configuration of `test.exs` we're actually not using `ex_fdbmonitor`, and instead\nusing `EctoFoundationDB.Sandbox`. Instead of creating a FoundationDB cluster, this sandbox\nmanages a single `fdbserver` process. The port is chosen automatically, and it is not\nnecessary for your application to inspect this port. The sandbox will create the directory\n`.erlfdb_sandbox`.\n\nAn alternative approach is to use `ExFdbmonitor.Sandbox`, which uses `:local_cluster` to\nstart additional BEAM VMs, and allows testing of multiple node configurations. You may\nreference the ExUnit tests for an example of how to do that. Using `EctoFoundationDB.Sandbox`\nis faster and simpler.\n\n#### Configure test.exs\n\nThis is an important step that will ensure `mix test` does not connect to a FoundationDB\ndatabase that is running on your system in the default location.\n\n```elixir\n# test.exs\nimport Config\n\nconfig :example_app, ExampleApp.Repo,\n  open_db: &EctoFoundationDB.Sandbox.open_db/1\n```\n\n#### Set up supporting files\n\nWe define an ExUnit Case to be used by any of your tests that require a database\nconnection. It's called `TenantCase` because it sets up a randomly generated\nFoundationDB Tenant so that each Case is fully isolated in the database, which\nensures that your tests can execute concurrently.\n\nYou are encouraged to adapt this Case based on the needs of your application, but\navoid using a common `tenant_id`, as this will break the isolation.\n\nAlso note that `EctoFoundationDB.Sandbox` will delete all existing data in the tenant\nat `checkout` time, and will delete the tenant entirely on `checkin`.\n\n```bash\nmkdir test/support\ntouch test/support/tenant_case.ex\n```\n\n```elixir\n# test/support/tenant_case.ex\ndefmodule ExampleApp.TenantCase do\n  use ExUnit.CaseTemplate\n  alias Ecto.UUID\n  alias EctoFoundationDB.Sandbox\n  alias ExampleApp.Repo\n\n  setup do\n    tenant_id = UUID.autogenerate()\n    tenant = Sandbox.checkout(Repo, tenant_id, [])\n\n    on_exit(fn ->\n      Sandbox.checkin(Repo, tenant_id)\n    end)\n\n    {:ok, [tenant_id: tenant_id, tenant: tenant]}\n  end\nend\n```\n\nThe following edits to mix.exs will ensure your new Case is compiled by mix.\n\n```elixir\n# mix.exs\ndef project do\n  [\n    app: :example_app,\n    # ...\n    elixirc_paths: elixirc_paths(Mix.env()),\n    # ...\n  ]\nend\n# ...\ndefp elixirc_paths(:test), do: [\"lib\", \"test/support\"]\ndefp elixirc_paths(_), do: [\"lib\"]\n```\n\nFinally, we can write our test, which simply runs a trivial transaction. Of course,\nthis is where you'll put test logic that is relevant to your application.\n\n```elixir\n# test/example_app_test.exs\ndefmodule ExampleAppTest do\n  use ExampleApp.TenantCase\n  test \"greets the world\", context do\n    tenant = context[:tenant]\n    assert :ok = ExampleApp.Repo.transaction(fn -> :ok end, prefix: tenant)\n  end\nend\n```\n\nYou should now be able to run `mix test` successfully. You can safely delete the `.erlfdb_sandbox` directory\nat any time throughout development.","title":"For `MIX_ENV=test` - ExampleApp","ref":"example-app.html#for-mix_env-test"},{"type":"extras","doc":"In production, you have a choice. It's likely that you'll want to host FoundationDB on infrastructure that\nis separated from your application (Conventional), but there are other configurations that might be\ninteresting to you (Radical).\n\n#### Conventional: connect to remote database\n\nSimply construct prod.exs like so:\n\n```elixir\n# config/prod.exs\nimport Config\n\nconfig :example_app, ExampleApp.Repo,\n  cluster_file: \"/etc/foundationdb/fdb.cluster\"\n```\n\nThe `:cluster_file` option should identify a file on the filesystem. Your application should have both read\nand write access to this file. The contents of this file are generated by the FoundationDB server and should not\nbe changed by hand. If you don't have this file, then you must copy it from the FoundationDB server.\n\nYou also must to ensure that your application instance has sufficient network access to the FDB servers.\n\n#### Radical: application manages its own database cluster\n\nAlternatively, your app's production config can bootstrap its own multi-node FoundationDB cluster. `ex_fdbmontior`\nmakes it easy to use the BEAM's clustering capabilities for node discovery. There are a couple reasons why you\nmight consider this approach.\n\n1. You simply want a local single-node FDB database, similar to a SQLite deployment.\n2. Your applicatiion needs multi-node ACID transactions, and you don't want the complexity or cost\n   associated with hosting a separate database.\n3. You run a deployment of your application that acts as the database and a deployment that acts as the\n   stateless application. Similar to the approach used by FLAME, you could create a `MIX_ENV=db`\n   variant of your app that runs `ex_fdbmonitor` and bootstraps the database, whereas `MIX_ENV=prod`\n   simply connects to it.\n4. You create an entirely separate Elixir app that is strictly concerned with running `ex_fdbmonitor`.\n   Once such an application is running, your main application can use the Conventional approach.\n\nIn any of these cases, your runtime.exs could resemble something like this:\n\n```elixir\n# config/runtime.exs\nimport Config\n\ndatabase_path = System.fetch_env!(\"DATABASE_PATH\")\n\nconfig :ex_fdbmonitor,\n  etc_dir: Path.join(database_path, \"etc\"),\n  run_dir: Path.join(database_path, \"run\")\n\nnode_idx = String.to_integer(System.fetch_env!(\"EXAMPLE_APP_NODE_IDX\"))\nnode_count = String.to_integer(System.fetch_env!(\"EXAMPLE_APP_NODE_COUNT\"))\ninterface = System.get_env(\"EXAMPLE_APP_COORDINATOR_IF\") || \"en0\"\n\naddr_fn = fn if ->\n   {:ok, addrs} = :inet.getifaddrs()\n\n   addrs\n   |> then(&:proplists.get_value(~c\"#{if}\", &1))\n   |> then(&:proplists.get_all_values(:addr, &1))\n   |> Enum.filter(&(tuple_size(&1) == 4))\n   |> hd()\n   |> :inet.ntoa()\n   |> to_string()\nend\n\nconfig :ex_fdbmonitor,\n  bootstrap: [\n    cluster:\n    if(node_idx > 0,\n      do: :autojoin,\n      else: [\n        coordinator_addr: addr_fn.(interface)\n      ]\n    ),\n    conf: [\n      data_dir: Path.join(database_path, \"data\"),\n      log_dir: Path.join(database_path, \"log\"),\n      fdbservers: [[port: 4500], [port: 4501]]\n    ],\n    fdbcli: if(node_idx == 0, do: ~w[configure new single ssd-redwood-1]),\n    fdbcli: if(node_idx == 2, do: ~w[configure double]),\n    fdbcli: if(node_idx == node_count - 1, do: ~w[coordinators auto])\n  ]\n```\n\nand prod.exs:\n\n```elixir\n# config/prod.exs\nconfig :livesecret, LiveSecret.Repo,\n  open_db: &ExFdbmonitor.open_db/1\n# ...\n```\n\nKeep in mind that `:erlang.nodes()` is used to detect nodes that any given node can join to in\norder to form the cluster on first boot. So your nodes must be able to reach each other.\n\nOn first bring-up, nodes should be started individually and serially.\n\nOnce a given node has been started, the `bootstrap` config, will be ignored on all subsequent restarts,\nin which `:etc_dir` and `:run_dir` are used.\n\nThe example config above uses a per-node index value to control the sequence of commands in the bootstrap,\nand assumes the nodes will start in the correct order. Feel free to use some other piece of data convenient\nto your deployment procedure.","title":"For `MIX_ENV=prod` - ExampleApp","ref":"example-app.html#for-mix_env-prod"},{"type":"extras","doc":"For completeness, we'll give our ExampleApp a Schema to write data to and give it an index via a Migration.\nWe suggest the reader consult the EctoFoundationDB documentation for details regarding Schemas, Indexes,\nand Migrations. We'll provide a minimal set-up here to get you started.\n\n#### Schema\n\nDefines a struct your app stores in the database.\n\n```elixir\n# lib/example_app/temperature_event.ex\ndefmodule ExampleApp.TemperatureEvent do\n  use Ecto.Schema\n  @primary_key {:id, :binary_id, autogenerate: true}\n\n  schema \"temperature_events\" do\n    field(:recorded_at, :naive_datetime_usec)\n    field(:kelvin, :float)\n    field(:site, :string)\n    timestamps()\n  end\nend\n```\n\n#### Indexes\n\nDefines the fields that can be used to efficiently access your data.\nRemember: all `EctoFoundationDB.Migration` modules must be included in your application release.\n\n```elixir\n# lib/example_app/migrations/temperature_event_indexes.ex\ndefmodule ExampleApp.Migrations.TemperatureEventIndexes do\n  use EctoFoundationDB.Migration\n  alias ExampleApp.TemperatureEvent\n\n  def change() do\n    [\n      create(index(TemperatureEvent, [:site])),\n      create(index(TemperatureEvent, [:recorded_at]))\n    ]\n  end\nend\n```\n\n#### Migration\n\nDefines the order in which the indexes are created and the corresponding monotonically increasing version numbers.\nRemember: your `EctoFoundationDB.Migrator` module must be included in your application release.\n\n```elixir\n# lib/example_app/repo.ex\ndefmodule ExampleApp.Repo do\n  # ...\n\n  use EctoFoundationDB.Migrator\n  def migrations() do\n    [{0, ExampleApp.Migrations.TemperatureEventIndexes}]\n  end\nend\n```\n\n#### Test\n\nFinally, we can write some data!\n\n```elixir\n# test/example_app_test.exs\ndefmodule ExampleAppTest do\n  use ExampleApp.TenantCase\n  alias ExampleApp.TemperatureEvent\n  alias ExampleApp.Repo\n\n  test \"greets the world\", context do\n    tenant = context[:tenant]\n\n    event =\n      Repo.insert!(\n        %TemperatureEvent{kelvin: 13.37, site: \"L2\", recorded_at: NaiveDateTime.utc_now()},\n        prefix: tenant\n      )\n\n    assert %TemperatureEvent{site: \"L2\"} = event\n\n    assert ^event = Repo.get_by!(TemperatureEvent, [site: \"L2\"], prefix: tenant)\n  end\nend\n```","title":"Schemas and Migrations - ExampleApp","ref":"example-app.html#schemas-and-migrations"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}